/*
  attack.c

  Dug Song <dugsong@anzen.com>

  Copyright (c) 1999 Anzen Computing. All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
  
  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  3. All advertising materials mentioning features or use of this software
     must display the following acknowledgement:
     This product includes software developed by Anzen Computing.
  4. Neither the name of Anzen Computing nor the names of its
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

  $Id: attack.c,v 1.8 1999/05/03 16:24:18 dugsong Exp $
*/

#include "config.h"

#ifdef STDC_HEADERS
#include <stdio.h>
#endif
#include <pcap.h>
#include "attack.h"
#include "sniff.h"
#include "frag.h"
#include "send.h"

int attack_type;
int attack_num;

char *attack_names[] = {
  "baseline", "frag", "tcp", "tcbc", "tcbt", "insert", "evade"
};

char *
attack_name(int type)
{
  return attack_names[type];
}

int
attack_baseline(u_char *u, struct pcap_pkthdr *hdr, u_char *pkt)
{
  return (send_packet(pkt + sniff_ll_len, hdr->len - sniff_ll_len));
}

int
attack_frag(u_char *u, struct pcap_pkthdr *hdr, u_char *pkt)
{
  FRAG *f, *frag = NULL;

  if (attack_num == 1) {
    frag = frag_make(pkt + sniff_ll_len, hdr->len - sniff_ll_len, 8);
  }
  else if (attack_num == 2) {
    frag = frag_make(pkt + sniff_ll_len, hdr->len - sniff_ll_len, 24);
  }
  else if (attack_num == 3) {
    frag = frag_make(pkt + sniff_ll_len, hdr->len - sniff_ll_len, 8);
    if ((f = frag_last(frag)) != NULL)
      frag_swap(f->prev);
  }
  else if (attack_num == 4) {
    frag = frag_make(pkt + sniff_ll_len, hdr->len - sniff_ll_len, 8);
    if ((f = frag_last(frag)) != NULL)
      frag_dup(f->prev);
  }
  else if (attack_num == 5) {
    int i;
    frag = frag_make(pkt + sniff_ll_len, hdr->len - sniff_ll_len, 8);
    if ((f = frag_last(frag)) != NULL)
      frag_dup(f->prev);
    /* XXX - lame. */
    for (i = 0, f = f->head ; f && f->next != NULL ; f = f->next, i++)
      if (i % 2) f = frag_swap(f);
  }
  else if (attack_num == 6) {
    frag = frag_make(pkt + sniff_ll_len, hdr->len - sniff_ll_len, 8);
    if ((f = frag_last(frag)) != NULL)
      frag_swap(f);
  }
  else if (attack_num == 7) {
    /* Fragment into 16-byte frags, and precede each frag with an
       8-byte null-data frag that overlaps the second half of it.
       This amounts to the 16-byte frag forward-overlapping the
       null-data back to the real attack. */
    frag = frag_make(pkt + sniff_ll_len, hdr->len - sniff_ll_len, 16);

    if (frag)
      frag_add_fwd_overwrite(frag);
  }
  else return 0;

  if (frag) {
    frag_send(frag->head);
    frag_free(frag->head);
    return 1;
  }
  return (attack_baseline(u, hdr, pkt));
}

void *
attack_init(int type, int num, char *ebuf)
{
  void *attack = NULL;
  
  if (type == ATTACK_BASELINE && num == 1) {
    attack = attack_baseline;
  }
  else if (type == ATTACK_FRAG && num < 8) {
    attack = attack_frag;
  }
  else {
    strcpy(ebuf, "attack unimplemented");
    return NULL;
  }
  attack_type = type;
  attack_num = num;
  
  return attack;
}
